"""
Nexom server settings.

This file is generated by Nexom buildTools.
"""

from __future__ import annotations

from pathlib import Path


# ======== module ========
# Project root directory
directory: str = "{pwd_dir}"

# ======== gunicorn ========
_address: str = "{g_address}"
_port: int = {g_port}
_workers: int = {g_workers}
_reload: bool = {g_reload}


def project_path(*parts: str) -> str:
    """
    Build an absolute path under the project directory.
    """
    return str(Path(directory, *parts))